
local AchievementIcon = "Interface\\AddOns\\Overachiever\\AchShield";
local tooltip_complete = { r = 0.2, g = 0.5, b = 0.2 }
local tooltip_incomplete = { r = 1, g = 0.1, b = 0.1 }

local function isCriteria(achID, name)
  local n, t, complete
  for i=1,GetAchievementNumCriteria(achID) do
    n, t, complete = GetAchievementCriteriaInfo(achID, i)
    if (n == name) then  return true, complete;  end
  end
end

--[[
local function isCriteria_hidden(achID, name)
  local n, t, complete
  local i = 0
  repeat
    i = i + 1
    n, t, complete = GetAchievementCriteriaInfo(achID, i)
    if (n == name) then  return true, complete;  end
  until (not n)
end
--]]

local lastreminder = 0
local SharedMedia = LibStub:GetLibrary("LibSharedMedia-3.0")

local function PlayReminder()
  if (Overachiever_Settings.SoundAchIncomplete ~= 0 and time() >= lastreminder + 15) then
    local sound = SharedMedia:Fetch("sound", Overachiever_Settings.SoundAchIncomplete)
    if (sound) then
      PlaySoundFile(sound)
      lastreminder = time()
    end
  end
end


-- UNIT TOOLTIP HOOK
----------------------

-- "To All The Squirrels I've Loved Before" achievement:
local ACH_LoveCritters_Complete
local function CritterCheck(name, unit)
  if (ACH_LoveCritters_Complete or not Overachiever_Settings.CritterTip_loved) then  return;  end
  if (name and unit and UnitCreatureType(unit) == OVERACHIEVER_STRINGS_CRITTER) then
    if (select(4, GetAchievementInfo(OVERACHIEVER_ACHID.LoveCritters))) then
      ACH_LoveCritters_Complete = true;
      return;
    end
    local isCrit, complete = isCriteria(OVERACHIEVER_ACHID.LoveCritters, name)
    if (isCrit) then
      return complete and OVERACHIEVER_STRINGS_ACH_LOVECRITTERS_COMPLETE or OVERACHIEVER_STRINGS_ACH_LOVECRITTERS_INCOMPLETE, complete
    end
  end
end

local RaceClassAch = {
  LetItSnow = { "LetItSnow_flaked", OVERACHIEVER_STRINGS_ACH_LETITSNOW_COMPLETE, OVERACHIEVER_STRINGS_ACH_LETITSNOW_INCOMPLETE },
  FistfulOfLove = { "FistfulOfLove_petals", OVERACHIEVER_STRINGS_ACH_FISTFULOFLOVE_COMPLETE, OVERACHIEVER_STRINGS_ACH_FISTFULOFLOVE_INCOMPLETE },
};

local function RaceClassCheck(ach, text)
  local id = OVERACHIEVER_ACHID[ach]
  if (select(4, GetAchievementInfo(id))) then
    RaceClassAch[ach] = nil;
    return;
  end
  local isCrit, complete = isCriteria(id, text)
  if (isCrit) then
    return complete and RaceClassAch[ach][2] or RaceClassAch[ach][3], complete
  end
end

function Overachiever.ExamineSetUnit(tooltip)
  tooltip = tooltip or GameTooltip  -- Workaround since another addon is known to break this
  local name, unit = tooltip:GetUnit()
  local text, complete = CritterCheck(name, unit)

  if (not text and UnitIsPlayer(unit)) then
    local raceclass = UnitRace(unit).." "..UnitClass(unit)
    for key,tab in pairs(RaceClassAch) do
      if (Overachiever_Settings[ tab[1] ]) then
        text, complete = RaceClassCheck(key, raceclass)
        if (text) then  break;  end
      end
    end
  end

  if (text) then
    local r, g, b
    if (complete) then
      r, g, b = tooltip_complete.r, tooltip_complete.g, tooltip_complete.b
    else
      r, g, b = tooltip_incomplete.r, tooltip_incomplete.g, tooltip_incomplete.b
      PlayReminder()
    end
    tooltip:AddLine(text, r, g, b)
    tooltip:AddTexture(AchievementIcon)
    tooltip:Show()
  end
end


-- WORLD OBJECT TOOLTIP HOOK
------------------------------

local WorldObjAch = {
  WellRead = { "WellReadTip_read", OVERACHIEVER_STRINGS_ACH_WELLREAD_COMPLETE, OVERACHIEVER_STRINGS_ACH_WELLREAD_INCOMPLETE },
  HigherLearning = { "WellReadTip_read", OVERACHIEVER_STRINGS_ACH_WELLREAD_COMPLETE, OVERACHIEVER_STRINGS_ACH_WELLREAD_INCOMPLETE },
  Scavenger = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE, true },
  OutlandAngler = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE, true },
  NorthrendAngler = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE, true },
};

local function WorldObjCheck(ach, text)
  local id = OVERACHIEVER_ACHID[ach]
  if (select(4, GetAchievementInfo(id))) then
    WorldObjAch[ach] = nil;
    return;
  end
  local isCrit, complete = isCriteria(id, text)
  if (isCrit) then
    return complete and WorldObjAch[ach][2] or WorldObjAch[ach][3], complete, WorldObjAch[ach][4]
  end
end

function Overachiever.ExamineOneLiner(tooltip)
-- Unfortunately, we couldn't find a "GameTooltip:SetWorldObject" or similar type of thing, so we have to check for
-- these sorts of tooltips in a less direct way.
  tooltip = tooltip or GameTooltip  -- Workaround since another addon is known to break this
  if (tooltip:NumLines() == 1) then
    local n = tooltip:GetName()
    if (getglobal(n.."TextRight1"):GetText()) then  return;  end
    local tiptext = getglobal(n.."TextLeft1"):GetText()
    local text, complete, angler
    for key,tab in pairs(WorldObjAch) do
      if (Overachiever_Settings[ tab[1] ]) then
        text, complete, angler = WorldObjCheck(key, tiptext)
        if (text) then  break;  end
      end
    end
    if (text) then
      local r, g, b
      if (complete) then
        r, g, b = tooltip_complete.r, tooltip_complete.g, tooltip_complete.b
      else
        r, g, b = tooltip_incomplete.r, tooltip_incomplete.g, tooltip_incomplete.b
        if (not angler or not Overachiever_Settings.SoundAchIncomplete_AnglerCheckPole or
            not IsEquippedItemType("Fishing Poles")) then
          PlayReminder()
        end
      end
      tooltip:AddLine(text, r, g, b)
      tooltip:AddTexture(AchievementIcon)
      tooltip:Show()
    end
  end
end


-- ITEM TOOLTIP HOOK
-----------------------

local FoodCriteria, DrinkCriteria, ItemLookupTabBuilt = {}, {}

--local lastitemTime, lastitemLink = 0

function Overachiever.BuildItemLookupTab()
-- Build lookup tables (since examining the criteria each time is time-consuming)
  if (ItemLookupTabBuilt) then  return;  end

  local achID, i, n = OVERACHIEVER_ACHID.TastesLikeChicken, 0
  repeat -- repeat/until used because these are "hidden" criteria: GetAchievementNumCriteria returns only 1.
    i = i + 1
    n = GetAchievementCriteriaInfo(achID, i)
    if (n) then  FoodCriteria[n] = i;  end
  until (not n)

  achID, i = OVERACHIEVER_ACHID.HappyHour, 0
  repeat
    i = i + 1
    n = GetAchievementCriteriaInfo(achID, i)
    if (n) then  DrinkCriteria[n] = i;  end
  until (not n)

  ItemLookupTabBuilt = true
end

local LBI = LibStub:GetLibrary("LibBabble-Inventory-3.0"):GetLookupTable()

local ConsumeItemAch = {
  TastesLikeChicken = { "Item_consumed", OVERACHIEVER_STRINGS_ACH_CONSUME_COMPLETE, OVERACHIEVER_STRINGS_ACH_CONSUME_INCOMPLETE, OVERACHIEVER_STRINGS_ACH_CONSUME_INCOMPLETE_EXTRA, FoodCriteria },
  HappyHour = { "Item_consumed", OVERACHIEVER_STRINGS_ACH_CONSUME_COMPLETE, OVERACHIEVER_STRINGS_ACH_CONSUME_INCOMPLETE, OVERACHIEVER_STRINGS_ACH_CONSUME_INCOMPLETE_EXTRA, DrinkCriteria },
};

local function ItemConsumedCheck(ach, text)
  local id = OVERACHIEVER_ACHID[ach]
  local achcomplete = select(4, GetAchievementInfo(id))
  if (achcomplete and not Overachiever_Settings[ ConsumeItemAch[ach][1].."_whencomplete" ]) then
    return;
  end
  local critNum = ConsumeItemAch[ach][5][text]
  if (critNum) then
    local n, t, complete = GetAchievementCriteriaInfo(id, critNum)
    return complete and ConsumeItemAch[ach][2] or achcomplete and ConsumeItemAch[ach][4] or ConsumeItemAch[ach][3], complete, achcomplete
  end
end

function Overachiever.ExamineItem(tooltip)
  tooltip = tooltip or this or GameTooltip  -- Workaround in case another addon breaks this
  local name, link = tooltip:GetItem() -- Issue: This doesn't reliably get the item we want!
  if (not link) then  return;  end
  -- Could check IsUsableItem(link) to see if it's something in your inventory..
  local itemMinLevel, itemType, subtype = select(5, GetItemInfo(link))
  if ((itemType == LBI["Consumable"] and (subtype == LBI["Food & Drink"] or subtype == LBI["Consumable"])) or
      (itemType == LBI["Trade Goods"] and subtype == LBI["Meat"])) then
    local text, complete, achcomplete
    for key,tab in pairs(ConsumeItemAch) do
      if (Overachiever_Settings[ tab[1] ]) then
        text, complete, achcomplete = ItemConsumedCheck(key, name)
        if (text) then  break;  end
      end
    end
    if (text) then
      local r, g, b
      if (complete) then
        r, g, b = tooltip_complete.r, tooltip_complete.g, tooltip_complete.b
      else
        r, g, b = tooltip_incomplete.r, tooltip_incomplete.g, tooltip_incomplete.b
        if (not achcomplete and tooltip == GameTooltip and itemMinLevel <= UnitLevel("player")) then
          -- Extra checks needed since the previous item sometimes shows up on the tooltip?
          PlayReminder();
        end
      end
      tooltip:AddLine(text, r, g, b)
      tooltip:AddTexture(AchievementIcon)
      tooltip:Show()
    end
  end
end


-- Register some Blizzard sounds
-----------------------------------

if (SharedMedia) then
  local soundtab = {
  ["Sound\\Doodad\\BellTollAlliance.wav"] = "Bell - Alliance",
  ["Sound\\Doodad\\BellTollHorde.wav"] = "Bell - Horde",
  ["Sound\\Doodad\\BellTollNightElf.wav"] = "Bell - Night Elf",
  ["Sound\\Doodad\\BellTollTribal.wav"] = "Drum Hit",
  ["Sound\\Doodad\\BoatDockedWarning.wav"] = "Bells - Boat Arrived",
  ["Sound\\Doodad\\G_GongTroll01.wav"] = "Gong - Troll",
  ["Sound\\Spells\\ShaysBell.wav"] = "Mellow Bells",

  ["Sound\\Spells\\PVPEnterQueue.wav"] = "Enter Queue",
  ["Sound\\Spells\\bind2_Impact_Base.wav"] = "Hearthstone Bind",
  ["Sound\\Doodad\\KharazahnBellToll.wav"] = "Karazhan Bell",

  ["Sound\\Interface\\AuctionWindowOpen.wav"] = "Auction Ding",
  ["Sound\\Interface\\AuctionWindowClose.wav"] = "Auction Bells",
  ["Sound\\Interface\\AlarmClockWarning1.wav"] = "Alarm Clock 1",
  ["Sound\\Interface\\AlarmClockWarning2.wav"] = "Alarm Clock 2",
  ["Sound\\Interface\\AlarmClockWarning3.wav"] = "Alarm Clock 3",
  ["Sound\\Interface\\MapPing.wav"] = "Map Ping",

  ["Sound\\Spells\\SimonGame_Visual_GameTick.wav"] = "Simon Ding",
  ["Sound\\Spells\\SimonGame_Visual_LevelStart.wav"] = "Simon Start Game",
  ["Sound\\Spells\\SimonGame_Visual_GameStart.wav"] = "Simon Start Level",

  ["Sound\\Spells\\YarrrrImpact.wav"] = "Yarrrr",
  }
  for data,name in pairs(soundtab) do
    SharedMedia:Register("sound", "Blizzard: "..name, data)
  end
  soundtab = nil
end
